@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2014	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -72,6 +72,7 @@ static unsigned long long int text_to_lsn(char *text);
 static RETSIGTYPE my_signal_handler(int sig);
 static RETSIGTYPE reload_config_handler(int sig);
 static void reload_config(void);
+static int get_query_result(int backend_id, char *query, POOL_SELECT_RESULT **res);
 
 #define CHECK_REQUEST \
 	do { \
@@ -86,6 +87,9 @@ static void reload_config(void);
 		} \
     } while (0)
 
+
+#define PG10_SERVER_VERSION	100000	/* PostgreSQL 10 server version num */
+
 /*
 * worker child main loop
 */
@@ -126,8 +130,7 @@ void do_worker_child(void)
 		/*
 		 * If streaming replication mode, do time lag checking
 		 */
-
-		if (pool_config->sr_check_period > 0 && MASTER_SLAVE && !strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP))
+		if (pool_config->sr_check_period > 0 && STREAM)
 		{
 			/* Check and establish persistent connections to the backend */
 			establish_persistent_connection();
@@ -195,9 +198,11 @@ static void discard_persistent_connection(void)
  */
 static void check_replication_time_lag(void)
 {
+	/* backend server version cache */
+	static int server_version[MAX_NUM_BACKENDS];
+
 	int i;
 	int active_nodes = 0;
-	POOL_STATUS sts;
 	POOL_SELECT_RESULT *res;
 	unsigned long long int lsn[MAX_NUM_BACKENDS];
 	char *query;
@@ -243,52 +248,42 @@ static void check_replication_time_lag(void)
 			return;
 		}
 
+		if (server_version[i] == 0)
+		{
+			query = "SELECT current_setting('server_version_num')";
+
+			/* Get backend serversion. If the query fails, keep previous info. */
+			if (get_query_result(i, query, &res) == 0)
+			{
+				server_version[i] = atoi(res->data[0]);
+				pool_debug("backend %d server version: %d", i, server_version[i]);
+			}
+		}
+			
 		if (PRIMARY_NODE_ID == i)
 		{
-			query = "SELECT pg_current_xlog_location()";
+			if (server_version[i] >= PG10_SERVER_VERSION)
+				query = "SELECT pg_current_wal_lsn()";
+			else
+				query = "SELECT pg_current_xlog_location()";
 		}
 		else
 		{
-			query = "SELECT pg_last_xlog_replay_location()";
+			if (server_version[i] >= PG10_SERVER_VERSION)
+				query = "SELECT pg_last_wal_replay_lsn()";
+			else
+				query = "SELECT pg_last_xlog_replay_location()";
 		}
 
-		sts = do_query(slots[i]->con, query, &res, PROTO_MAJOR_V3);
-		if (sts != POOL_CONTINUE)
-		{
-			if (res)
-				free_select_result(res);
-			pool_error("check_replication_time_lag: %s failed", query);
-			return;
-		}
-		if (!res)
-		{
-			pool_error("check_replication_time_lag: %s result is null", query);
-			return;
-		}
-		if (res->numrows <= 0)
-		{
-			pool_error("check_replication_time_lag: %s returns no rows", query);
-			free_select_result(res);
-			return;
-		}
-		if (res->data[0] == NULL)
-		{
-			pool_error("check_replication_time_lag: %s returns no data", query);
-			free_select_result(res);
-			return;
-		}
-
-		if (res->nullflags[0] == -1)
-		{
-			pool_log("check_replication_time_lag: %s returns NULL", query);
-			free_select_result(res);
-			lsn[i] = 0;
-		}
-		else
+		if (get_query_result(i, query, &res) == 0)
 		{
 			lsn[i] = text_to_lsn(res->data[0]);
 			free_select_result(res);
 		}
+		else
+		{
+			lsn[i] = 0;
+		}
 	}
 
 	for (i=0;i<NUM_BACKENDS;i++)
@@ -388,3 +383,46 @@ static void reload_config(void)
 		load_hba(get_hba_file_name());
 	reload_config_request = 0;
 }
+
+/*
+ * Execute query against specified backend.
+ * Return -1 on failure or 0 otherwise.
+ * Caller must prepare memory for POOL_SELECT_RESULT and pass it as "res".
+ */
+
+static 	int get_query_result(int backend_id, char *query, POOL_SELECT_RESULT **res)
+{
+	int sts = -1;
+
+	do_query(slots[backend_id]->con, query, res, PROTO_MAJOR_V3);
+
+	if (!res)
+	{
+		pool_error("Failed to check replication time lag. Query to node (%d) returned no result for node", backend_id);
+		return sts;
+	}
+
+	if ((*res)->numrows <= 0)
+	{
+		free_select_result(*res);
+		pool_error("Failed to check replication time lag. Query to node (%d) returned result with no rows", backend_id);
+		return sts;
+	}
+
+	if ((*res)->data[0] == NULL)
+	{
+		free_select_result(*res);
+		pool_error("Failed to check replication time lag. Query to node (%d) returned no data", backend_id);
+		return sts;
+	}
+
+	if ((*res)->nullflags[0] == -1)
+	{
+		free_select_result(*res);
+		pool_error("Failed to check replication time lag. Query to node (%d) returned NULL data", backend_id);
+		return sts;
+	}
+
+	sts = 0;
+	return sts;
+}
